import time
from datetime import datetime
from typing import List, Dict, Any

from bs4 import BeautifulSoup
from requests import HTTPError, ConnectionError, Session

from .exceptions import NetworkException
from .models import MediaResult, Image


class Nitter:
    def __init__(self, timeout=10, proxies=None) -> None:
        self.timeout = timeout

        self._session = Session()
        self._session.proxies = proxies

        self._base_url = 'https://nitter.moomoo.me'

    def format_url(self, url: str) -> str:
        return f'{self._base_url}{url}'

    def get_media(self, username: str) -> List[MediaResult]:
        media = []

        data = self._make_request(f"{username}/media")
        bs4 = BeautifulSoup(
            data,
            'html.parser'
        )

        for item in bs4.find_all('div', {'class': 'timeline-item'}):
            caption = item.find_next('div', {'class': 'tweet-content media-body'}).text

            tweet_date = item.find_next('span', {'class': 'tweet-date'})
            tweet_date = datetime.fromtimestamp(
                time.mktime(
                    time.strptime(
                        tweet_date.find('a')["title"],
                        '%b %d, %Y · %I:%M %p UTC'
                    )
                )
            )

            images = []
            for image in item.find_all('div', {'class': 'attachment image'}):
                images.append(
                    Image(
                        self.format_url(image.find_next('a', {'class': 'still-image'})['href'])
                    )
                )

            media.append(
                MediaResult(
                    caption,
                    images,
                    tweet_date
                )
            )

        return media

    def _make_request(self, path: str, params: Dict[str, Any] = None) -> str:
        try:
            req = self._session.get(
                f"{self._base_url}/{path}",
                params=params,
                timeout=self.timeout,
            )
            return req.text
        except (ConnectionError, HTTPError):
            raise NetworkException(f"Cannot connect to Nitter")



from os import path

from setuptools import setup, find_packages

current_dir = path.abspath(path.dirname(__file__))
with open(path.join(current_dir, 'README.md')) as f:
    description = f.read()

setup(
    name='nitter',
    packages=find_packages(),
    version='0.0.5',
    license='MIT',
    description='Nitter is a simple library to scrap Twitter data using the Nitter website',
    long_description=description,
    long_description_content_type='text/markdown',
    author='Ali Fotouhi',
    author_email='the.alif.dev@gmail.com ',
    url='https://github.com/iAliF/Nitter/',
    download_url='https://github.com/iAliF/Nitter/archive/refs/tags/v0.0.5-alpha.zip',
    install_requires=['requests', 'beautifulsoup4'],
    keywords=[
        'Nitter', 'Twitter', 'Twitter Scraper'
    ],
    project_urls={
        'Documentation': 'https://github.com/iAliF/Nitter/',
        'Source': 'https://github.com/iAliF/Nitter/',
        'Bug Tracker': 'https://github.com/iAliF/Nitter/issues',
    },
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Programming Language :: Python :: 3.13',
    ]
)



from dataclasses import dataclass
from datetime import datetime
from typing import List, Optional


@dataclass
class Image:
    src: str

    def __str__(self) -> str:
        return self.src

    def __repr__(self) -> str:
        return f"Image({self.src})"


@dataclass
class MediaResult:
    caption: Optional[str]
    images: List[Image]
    pub_date: datetime
